function Cost=fun_control_setup(Plant,Cost)
% FUN_CONTROL_SETUP   Generate GCM costs using Euclidean norm on output
%
% Define costs by:
%   Cost = gcm_hybrid_control_setup(Plant,<Cost>)
% Sets up fields in Cost(q), q=1,2,...,Q
%   R               instantaneous costs [S,A]
%   controllertype  {'optimal','predictive'}
%   gam             for 'controllertype'='optimal' [1,1]
%   Hp              for 'controllertype'='predictive' [1,1]


% Sets up the name of this file.
% =====================================================
%Cost(1).control_setup_file = mfilename;
if nargin==0
    return;
end

% Define instantaneous costs R(s,a) for each set point q:
% =======================================================
% R(s,a) is the cost for being in state s and using action a
Xref = Plant.Xref; 
Uref = Plant.Uref; 
Wref = Plant.Wref;
funh = Plant.funh;
[S,A,Q,M,n_x,n_u,n_y,n_w] = gcmdisp(Plant);

% Compute Euclidean costs in measurement
% --------------------------------------
% - initialize R's
for q=1:Q
    for a=1:A
        Cost(q).R = zeros(S,A);
    end
end
% - evaluate plant state s (at time k)
[n_x,S] = size(Plant.Xref);
Yc = feval(funh,Xref(:,1:S));
for q=1:Q
    
    % -deviation between q'th setpoint and measurement of all S states [S,1]
    J = gcm_eucl_norm(Wref(:,q),Yc)';
    
    VERYLARGE = 10*ceil(max(J(isfinite(J))));
    J = min(J,VERYLARGE);
    
    if 0
        % -assume no cost for control
        for a=1:A
            Cost(q).R(:,a) = J;
        end
    else
        % - control cost
        for a=1:A
%            Cost(q).R(:,a) = J + 0.1*abs(Uref(a));
            Cost(q).R(:,a) = J + 0.0*abs(Uref(a));
        end        
    end
end


% Controller type and parameters
% ==============================
% Set controller to 'predictive' or 'optimal'
if 1
    for q=1:Q, Cost(q).controllertype = 'optimal'; end
    % - set controller parameter
    for q=1:Q, Cost(q).gam = 0.98; end  % for optimal controller
else
    for q=1:Q, Cost(q).controllertype = 'predictive'; end
    % - set controller parameter
    for q=1:Q, Cost(q).H_p = 50; end    % for predictive controllerend
end


