function Sim = gcm_hybrid_simulation_setup(Plant)
% Set up simulation details
%
%   Sim = fun_simulation_setup(Plant)
%
% Fields in Sim:
%   WW  controller set points [K,n_y]
%   x0  plant initial state [n_x,1]
%   y0  plant initial output [n_y,1]
%   p_x0 initial state distribution, sparse [S,1]


% Read inputs
Wref = Plant.Wref;
Xref = Plant.Xref;
Ts = Plant.Ts;
[n_w,lW] = size(Wref);
[n_x,S] = size(Xref);

lW = length(Plant.Wref);
nsteps = 10;
lstep = 50;
WW = [];
for ns =1:lW
    % -go through all 
    w = Wref(:,ns)';
    for i=1:lstep
        WW = [WW;w];
    end
end
for ns = 1:lW
    % - take a set point randomly
    s = round((0.9999*lW*rand(1)+0.5+eps));
    w = Wref(:,s)';
    % - step of length lstep;
    for i=1:lstep
        WW = [WW;w];
    end
end

% Plant initial state
% - read from FUNF and FUNH
[tmp,x0] = eval(Plant.funf);
y0 = eval(Plant.funh);
p_x0 = sparse(S,1); s=gcm_x2s(x0,Xref); p_x0(s)=1;
            
% Set up output argument
Sim.WW = WW;
Sim.x0 = x0;
Sim.y0 = y0;
Sim.p_x0 = p_x0;


% ----------------------------------
function WWr = ramppi(ramp_length_t,Ts,w0,wf)
krl = ceil(ramp_length_t/Ts);
WWr = (wf-w0)*[0:krl]'/krl+w0;
    
