function idx=gcm_chbase(v,vmax)
% GCM_CHBASE Change from base vmax to base 10
%	idx = gcm_chbase(v,vmax)
%	v = gcm_chbase(idx,vmax)
% vmax  a row vector giving the # elements in each basis direction
% v     a row-matrix, where each row describes the vector in basis vmax
% idx   a row-matrix, in base 10 (corresponds to vmax=[10 10 10...]
%
% Following the MATLAB style of indexing, indexes are counted from 1.
% Example. Let the basis be given by vmax = [3 3 2] (e.g., a 3x3x2 dimensional cube)
%    Then  gcm_chbase(1,vmax) gives [1 1 1]
%           gcm_chbase(2,vmax) gives [1 1 2]
%           gcm_chbase(3,vmax) gives [1 2 1]
%               :
%          gcm_chbase(17,vmax) gives [3 3 1]
%           gcm_chbase(18,vmax) gives [3 3 2]
%    Upper limit is not implemented, so
%           gcm_chbase(19,vmax) will give [4 1 1]
%    The inverse mapping is obtained when length of v and vmax is the same, e.g.
%            gcm_chbase([3 3 1],vmax) gives 17

% Enso 11/2002, vectorized 11/2006

[r,lv] = size(v);
if lv==length(vmax)
   % convert from vector to "index"
	v = v(:,lv:-1:1);
	vmax = vmax(lv:-1:1);	
	vbar = v-1;
   %tmp1 = cumprod([1 vmax(1:lv-1)]);
   tmp1 = [1 cumprod(vmax(1:lv-1))];
   %idx = sum( tmp1.*vbar ) +1;
   idx = (tmp1*vbar'+1)';
else
    for ir = 1:r        % ei jaksa vektoroida tt kun ei ole pakko..
        % convert from index to vector
        tot = v(ir)-1;
        lvmax = length(vmax);
        for i=1:lvmax
            base = prod(vmax(i+1:lvmax));
            kerr = floor(tot/base);
            %if length(kerr)~=1, keyboard; end
            idx(ir,i)=kerr+1;
            tot = tot-kerr*base;
        end
    end
end
