function [YN,In]=gcm_check_for_grid(Xref);
%GCM_CHECK_FOR_GRID Check if Xref is built from a regular grid
%
%   [YN,In]=gcm_check_for_grid(Xref,SINK);
%
% YN    1=yes (is from a grid) or 0=no (is not from a grid)
% In    An array where each cell contains the input grid vector
%       In{1}   grid points for the 1'st dimension
%       In{2}   grid points for the 2'nd dimension
%       :
%       In{n_x}   grid points for the n_x'th dimension
%

% Enso 10/2006

[n_x,S]=size(Xref);

% check for sink cell
FLAG_SINK = 0; if ~all(all(isfinite(Xref))), FLAG_SINK = 1; end;
if FLAG_SINK, S = S-1; Xref=Xref(:,1:S); end; % remove sink cell (the S'th cell)

% Get partitions
for i=1:n_x
    In{i} = gcm_get_partitions(Xref(i,:));
    ni(i) = length(In{i});
end

% Check if from a grid
% -----------------------------------
% TEST 1: if # reference points equals product of individual points, 
% assume that these come from the grid..
if prod(ni)==S, YN=1; 
else, YN=0; 
end

% TEST 2: if two same vectors are next to each other, not a grid
if sum(sum(diff(Xref')')==0), YN=0; end
% TEST3: ?? It is still possible that the above tests fail..