function Xml = gcm_dispref(ss,Xref)
% GCM_DISPREF   Display values for a set of reference points
%
%       gcm_dispref(Xref)
%       Xml = gcm_dispref(ss,Xref)
%
% ss    indexes in Xref, 1 <= ss(i) <= S
% Xref  list of all reference points [n_x,S]
% Xml   maximum likelihood reference point
%
% If no output argument is given, a text display  ot the 
% distribution of Xref(:,ss) is produced in the command window.

if nargin==1
    Xref = ss;
    [n_x,S] = size(Xref);
    ss = [1:S];
end

[n_x,S] = size(Xref);
if nargout==0, fprintf('A set of %d states:\n',length(ss)); end
for i=1:n_x

    % find different values
    xdist = gcm_get_partitions(Xref(i,ss),0);
    
    % ML state
    count = [];
    for j=1:length(xdist)
        count(j) = length(find(Xref(i,ss)==xdist(j)));
    end
    iml = find(count==max(count)); %iml=iml(1);
    %Xml(i,1)=xdist(iml);
    Xml(i,1)=xdist(iml(1));
    
    % create a string to display
    str=[]; 
    for j = 1: length(xdist)
        if any(j==iml)
            str = [str,sprintf(' %5g*,',xdist(j))]; 
        else
            str = [str,sprintf(' %5g ,',xdist(j))]; 
        end
    end; 
    str = str(1:length(str)-1);
    
    % display
    if nargout==0, fprintf('x(%d) = {%s }\n',i,str); end
end
