function [ipr,ipt,ipa,V,D]=gcm_eig(P,K,Sigma)
% GCM_EIG   Find indexes to recurrent, transient and absorbing cells
%
% [ipr,ipt,ipa,V,D]=gcm_eig(P,K,Sigma)
%
% ipr   indexes to recurrent cells
% ipt   indexes to transient cells
% ipa   indexes to absorbing cells
% V,D   K eigenvectors and eigenvalues
% K     # eigenvectors to find (default K=100)
% Sigma look for eigenvalues closest Sigma (default: Sigma=1+100*eps);

% Compute eigenvectors & values: V, D
% ------------------------------------
% Compute eigenvectors using either EIGS or EIG
s = length(P);
if s>10
    % - Set EIGS parameters
    OPTS.disp = 0; OPTS.isreal = 1;
    if nargin<2, K=100; end
    if nargin<3, Sigma = 1+100*eps; end    % Sigma=1+100*eps seems to be faster than Sigma='LM';
    % - run EIGS
    [V,D,flag] = eigs(P,K,Sigma,OPTS);
else % for very small matrices
    [V,D] = eig(full(P));
    flag = 0; K=s;
end
if flag, beep; fprintf('\n    GCM_EIG: not all vectors converged (see HELP EIGS).'); end  

% Find absorbing/recurrent/transient states: ipa, ipr, ipt
% ---------------------------------------------------------
%fprintf('   Looking for recurrent (stationary) states in probability distribution...');
% - find states that are 100% recurrent for sure: p_jj = 1 (absorbing)
ipa = find(diag(P)==1);
% find eigenvectors with length 1
D = round(1000*D)/1000;     % length 1 indicates a recurrent direction
V = round(1000*V)/1000;     % nonzero elements indicate recurrent cells
id = find(diag(D)==1);

% find corresponding elements in probability distribution
if length(id)==1
    isp = find(V(:,id));
    sp = zeros(s,1); sp(isp)=1;
else
    sp = sum(abs(V(:,id))')';
end

% add 100% sure states
sp0 = zeros(s,1); sp0(ipa)=1;
sp = sp+sp0;

if nargout<4
    % - clear space for memory
    clear V D sp0
end

ipr = find(sp);    % indexes of p_a (absorbing states)
ipt = find(~sp);  % indexes to p_t (transient states)

