function en = gcm_eucl_norm(z,zz,W)
% GCM_EUCL_NORM Compute Euclidean norm between two matrices
%
%   en = gcm_eucl_norm(z,zz,W)
%
%   z    set of I-dimensional vectors, [I,S] 
%   zz   another set of I-dimensional vectors, [I,SS]
%   W    diagonal weighting matrix <default W=I>, [I,I]
%   en   Euclidean distance between i'th column vector in z 
%        and j'th column vector in zz, matrix [S,SS]
%   Efficient if SS>S

% Enso 2006


[I,S]=size(z);
if nargin<3
    W=eye(I,I);
end
for s=1:S   % for each vector in z
    for i=1:I   % for each element in vector
        %e2(i,:) = [z(i,s)-zz(i,:)].^2;
        e2(i,:) = (W(i,i)*[z(i,s)-zz(i,:)]).^2;
    end
    if i>1
        se2 = sum(e2);  % \Sigma_i[(z(i,s)-zz(i,ss))^2]
    else
        se2 = e2;
    end
    en(s,:) = sqrt(se2);
end
