function gcm_free(i1,i2,i3,i4,i5,i6)
% GCM_FREE Simulate free trajectory from an initial state
%
% gcm_free(Plant_cl,p0,q,<T>,<ilist>)
% gcm_free(Plant,Cost,p0,q,<T>,<ilist>)
%
% Plant_cl  Closed-loop plant-structure, Plant_cl.f(1).P is used
% p0        initial state, either an index s, or a probability distribution [S,1]
% q         Set point controller to consider.
% T         is a final time T=tf [1,1] or time vector T=[t1,tf], or T=[t1:Ts:tf]
%           T is in real time, with sampling time given in Plant.Ts.
% ilist     list of states x to plot

if isstruct(i2)
    % Plant & Cost given => generate closed loop map
    Plant_cl = gcmclosed(i1,i2);
    p0 = i3;
    q = i4;
    if nargin>4, T=i5;
    else, T=[];
    end
    if nargin>5, ilist = i6;
    else, ilist=[1:size(Plant_cl.Xref,1)];
    end
else
    % Closed-loop map is given
    Plant_cl = i1;
    p0 = i2;
    q = i3;
    if nargin>3, T=i4;
    else, T=[];
    end
    if nargin>4, ilist = i5;
    else, ilist=[1:size(Plant_cl.Xref,1)];
    end
end

gcm_step(Plant_cl,p0,q,T,ilist);
