function [Zref,ns] = gcm_inigrid(In1,In2,In3,In4,In5,In6,In7,In8,In9)
% GCM_INIGRID   Initialize a grid (lattice) structure
%   Xref = gcm_inigrid(In1,In2,...,InX)
% Inx       Discretization of xth input [1,Vx]
% Xref      Matrix containing reference points [n_X,S]
% ns        Vector of grid dimensions [In1 In2 ... InX]

% dimension of state space:
ns = [];
X = nargin;
for x=1:X
    varname = sprintf('In%d',x);
    ns = [ns size(eval(varname),2)];
end
disp(sprintf('GCM_INIGRID: Initializing a %dx%d grid..',prod(ns),X))

% all combinations of inputs:
idx =  gcm_allpnts_int(ns);
[S,X]=size(idx);

% corresponding reference points
t0 = clock; t00 = clock; DISP=1;
Zref = zeros(X,S);

% Generate reference points
for x=1:X
    for v=1:ns(x)
        zi = find(idx(:,x)==v);
        val = eval(sprintf('In%d(%d)',x,v));  
        Zref(x,zi) = val;
    end
    if (etime(clock,t0)>15 & DISP) | (etime(clock,t0)>3*60)
        done = x/X;
        undone = 1-x/X;
        dt = etime(clock,t00)/done;
        disp(sprintf('GCM_INIGRID: %3.2f%% done, %1.0f min to go..',100*done,undone*dt/60));
        DISP=0; t0 = clock;
    end
end
disp('GCM_INIGRID: ..ok.')