function gcm_plotx(Ts,xref,XX,YY,ZZ)
% GCM_PLOTS Plots one or two vectors of sampled values
% Partitioning is shown at the y-axes ticks
%   gcm_plotx(Ts,xref,XX)
%   gcm_plotx(Ts,xref,XX,YY)
%   gcm_plotx(Ts,xref,XX,YY,ZZ)
%
%   Ts         sampling time
%   xref       partitioning
%   XX,YY      column vector(s)

xref = xref(isfinite(xref));
[K,n_x]=size(XX);
kk = [1:K]';
tt = Ts*kk;
% plot
switch(nargin-2)
case 1
    [xx,yy]=stairs(tt,XX(kk,:));
    plot(xx,yy,'b-');
case 2
    [xx,yy]=stairs(tt,XX(kk,:));
    [xx2,yy2]=stairs(tt,YY(kk,:));
    dy = 0.001*(max(YY)-min(YY))+0.00001*max(YY);
    h=plot(xx2,yy2+dy,'r-',xx,yy,'b-');
    set(h(2),'linewidth',2);
case 3
    [xx,yy]=stairs(tt,XX(kk,:));
    [xx2,yy2]=stairs(tt,YY(kk,:));
    [xx3,yy3]=stairs(tt,ZZ(kk,:));
    dy = 0.001*(max(YY)-min(YY))+0.00001*max(YY);
    dz = 0.001*(max(YY)-min(YY))-0.00001*max(YY);
    h=plot(xx3,yy3+dz,'k:',xx2,yy2+dy,'r-',xx,yy,'b-');
    set(h(3),'linewidth',2);
end

% set y-axes
pv = gcm_get_partitions(xref);
if length(pv)>0 % 1
    set(gca,'ytick',pv);
    set(gca,'yticklabel',pv);
    if length(pv)>10
        str=get(gca,'yticklabel');
        rstr = length(pv);
        for i=1:rstr
            if i==1 | i==rstr | i==ceil(rstr/2), ; 
            else, str(i,:)=' '*ones(1,size(str,2));
            end
        end
        set(gca,'yticklabel',str);
    end
end
maxx = max(max(XX(kk)),max(pv));
minx = min(min(XX(kk)),min(pv));
dx = 0.02*(maxx-minx)+0.01*maxx;
% set axes
axis([0 Ts*(K+1) minx-dx maxx+dx]);

