function Xrnd = gcm_sampleu(Xref,In)
% GCM_SAMPLEU Generate random samples from the supporting hypercube
% for all vectors in Xref. Assumes a grid.
%       Xrnd = gcm_sampleu(Xref,In)
% Xref  a set of column vectors
% In    a cell array 
%       In{1} partitioning for the 1'st dimension, a row vector
%       :

% Enso 10/2006
[n_x,S] = size(Xref);

if nargin<2
    for i=1:n_x
        In{i} = gcm_get_partitions(Xref(i,:)); % part. for i'th component of state
    end
end

% generate a full random grid
for i=1:n_x
    for j=1:length(In{i})
        z = In{i}(j);
        [minx,maxx] = minmax(In{i},z);
        idx = find(Xref(i,:)==z);
        Xrnd(i,idx) = (maxx-minx)*rand(1,length(idx))+minx;
    end
end

%%%%%%%%%%%%%%%%%%%%% SUBFUN MINMAX %%%%%%%%%%%%%%%%%%
function [minX,maxX]=minmax(pv,s)
% Find min & max boundaries for the support interval the data belongs to.
% pv    set reference points (vector)
% s     input data sample
lpv = length(pv);
if lpv==1
    minX = pv; maxX = pv;
    return;
end
[ans,is] = min(abs(s-pv));
if is>1 & is<lpv
    % interval between two centers
    maxX = (pv(is) + pv(is+1)) / 2;
    minX = (pv(is-1) + pv(is)) / 2;;
elseif is==1
    % first discrete value
    maxX = (pv(is) + pv(is+1)) / 2;
    minX = 1.5*pv(is) - 0.5*pv(is+1);
elseif is==lpv;
    % last discrete value
    maxX = 1.5*pv(is) - 0.5*pv(is-1);
    minX = (pv(is-1) + pv(is)) / 2;
end

