function pp = gcm_step(Plant,p0,a,T,ilist)
% GCM_STEP Simulate and plot a discrete state trajectory
%
% p_x = gcm_step(Plant,s,a,<T>,<ilist>)
% p_x = gcm_step(Plant,p0,a,<T>,<ilist>)
%
% p0    initial state, either an index s, or a probability distribution [S,1]
% Plant Plant-structure, Plant.f(a).P is used
% T     is a final time T=tf [scalar] or time vector T=[t1,tf], or T=[t1:Ts:tf]
%       T is in real time, sampling time is given in Plant.Ts
%       If T is not given, it is determined automatically.
% ilist is a list of states x(i) to plot, default: all
% Figures are plotted as functions of samples (not T).

% Enso 11/2006

if nargin<3, error('Not enough input arguments'); end
if nargin<5, 
    n_x = length(Plant.Xref(:,1));
    ilist = [1:n_x];
end
if nargin<4
    k0 = 0; kf=[]; T=[];
else
    if length(T)==0
        k0 = 0;
        kf=[];
    elseif length(T)==1, 
        k0 = 0; 
        kf = ceil(T/Plant.Ts);
    elseif length(T)==2, 
        k0 = floor(T(1)/Plant.Ts); 
        kf = ceil(T(2)/Plant.Ts);
    else
        k0 = floor(T(1)/Plant.Ts);
        kf = ceil(T(length(T))/Plant.Ts);
    end
end

DSP=0;

if DSP, fprintf('Simulating trajectories...\n'); end

% disp s & a:
if length(p0)==1, ss=p0;
else, ss = find(p0); ss = ss(1:min(8,length(ss))); 
end

if DSP
    for s=ss
        disp(sprintf('x(%d) = ',s))
        disp(Plant.Xref(:,s)');
    end
    disp(sprintf('u(%d) = ',a))
    disp(Plant.Uref(:,a)');
end

% simulate:
% - pp has kf+1 columns
pp = gcm_simp(Plant.f(a).P,p0,kf);
% resample, if necessary, according to T
if length(kf)==0, kf=size(pp,2)-1; end
if length(T)>2, 
    kk = T/Plant.Ts;
    pp = pp(:,kk+1);
else
    pp = pp(:,[k0:kf]+1);
end

% Plot
[I,S] = size(Plant.Xref);
li = length(ilist);
switch(li)
case 1
    Ix = 1; Iy = 1;
otherwise
    Ix = ceil(li/2);
    Iy = 2;
end
ii = 0;
for i=ilist
    xref = Plant.Xref(i,:);
    xref = xref(isfinite(xref));
    if i==1, if DSP, fprintf('ok. Projecting...'); end, end
    ppp = gcm_project(pp,xref);
    ii = ii+1;
    if ~(Ix==1 & Iy==1), subplot(Ix,Iy,ii); end
    tt = [k0:kf]*Plant.Ts;
    gcm_pplot(ppp,gcm_get_partitions(xref),tt);
    is_bottom_row = (i-Iy*(Ix-1))>0;
    xlabel(sprintf('t (T_s=%g)',Plant.Ts));
    if isfield(Plant,'Xname')
        str = Plant.Xname{i};
    else
        str = sprintf('p(x_{%d})',i);
    end
    ylabel(str)
    gcm_ff;
end

fprintf('\ok.');
gcm_ff;
if nargout==0, clear pp; end