function Xk1 = gcm_vodesolver(funhan,Xk,Uk,Ts)
% GCM_VODESOLVER Vectorized ode-solver
%
%   Xk1B = gcm_vodesolver(function_handle,XkB,Uk,Ts)
%
% Solves ode in multiple batches. XkB is a matrix of size [n_X,B]
% where each column contains an input state. It seems to be faster
% to solve equations in relatively small batches, which is what
% this function does. The function shoud accept vector inputs Xk
% of length n_X*B2, where B2 is the 'optimal' batch size.

% EI 9/2007

[n_X,B] = size(Xk);
% 'optimal' batch size (experimental quess..)
Bopt = ceil(80/n_X);

% convert x from a matrix to a long vector
XkV = gcm_v2v(Xk);

% pass to ode-solver
div = ceil(B/Bopt);

while 1
    try
        Bnow = ceil(B/div);
        Bv = Bnow*n_X;      % batch size in the long vector XkV
        for b = 1:div
            %[b div Bv]
            % indexes to batch elements in the long vector
            ii = (b-1)*Bv+1 : min(B*n_X,b*Bv); 
            % solve ode
            [T,X]=ode23t(funhan,[0 Ts],XkV(ii),[],Uk);
            % read result
            Xk1V(ii,1) = X(length(T),:)';
        end
        break;
    catch
        % in case of error, assume that it was due to out-of-memory => reduce batch size
        %div = 2*div;
        %fprintf('/');
        %if div>=(B/2), 
        error(sprintf('GCM_VODESOLVER: %s',lasterr)); 
        %end
    end
end

% convert x from a vector to a matrix
Xk1 = gcm_v2v(Xk1V,n_X);
