function Cost = gcm_ccp(Cost,i1,Qlist)
% GCM_CCP   Change controller parameters
%
%   Cost = gcmccp(Cost,gamma,<qlist>)
%   Cost = gcmccp(Cost,H_p,<qlist>)


Q = length(Cost);
if nargin<3
    Qlist = [1:Q];
end

if i1<1, CT = 1; % optimal
else, CT = 2;  % predictive
end

% change structure fields, if necessary
% => all Q controllers must be either optimal or predictive
switch(CT)
case 1
    if isfield(Cost,'H_p'), 
        Cost = rmfield(Cost,'H_p'); 
        fprintf('GCM_CCP: Changing to optimal control.\n')
        % previous Jstars are not valid anymore
        if isfield(Cost,'Jstar'), Cost = rmfield(Cost,'Jstar'); end
    end
case 2
    if isfield(Cost,'gam'), 
        Cost = rmfield(Cost,'gam'); 
        fprintf('GCM_CCP: Changing to predictive control.\n')
        % previous Jstars are not valid anymore
        if isfield(Cost,'Jstar'), Cost = rmfield(Cost,'Jstar'); end
    end
end

for q=Qlist
    switch(CT)
    case 1
        % optimal controller
        Cost(q).controllertype = 'optimal';
        Cost(q).gam = i1;
    case 2
        % predictive controller
        Cost(q).controllertype = 'predictive';
        Cost(q).H_p = i1;
    end
end

gcmdisp(Cost);