function [S,A,Q,M,n_x,n_u,n_y,n_w]=gcmdisp(STR)
% GCMDISP Display info on plant discretization to command window
%
%   gcmdisp(Plant)
%   gcmdisp(Cost);
%
% The following commands do not display, but return output arguments:
%   [S,A,Q,M,n_x,n_u,n_y,n_w] = gcmdisp(Plant)
%   [S,A,Q] = gcmdisp(Cost)

% Enso 11/2006

ISPLANT = 0; ISCOST=1;
if isfield(STR,'Xref')
    ISPLANT = 1;
elseif isfield(STR,'E')
    ISCOST = 1;
end

if ISPLANT
    Wref = STR.Wref;
    Yref = STR.Yref;
    Xref = STR.Xref;
    Uref = STR.Uref;
    
    [n_w,Q] = size(Wref);
    [n_y,M] = size(Yref);
    [n_x,n_Xref] = size(Xref);
    if isfield(STR,'sref'), S = max(STR.sref);
    else, S = n_Xref; 
    end
    [n_u,A] = size(Uref);
    if nargout, return; end
    
    disp('== MODEL INFO ==============================================')
    disp(sprintf('- state equations: ''%s''',STR.funf))
    disp(sprintf('- measurement equations: ''%s''',STR.funh))
    
    disp('== DISCRETIZATION ==========================================')
    disp(sprintf('Sampling time: %g',STR.Ts));
    
    disp(sprintf('References w: Q=%d',Q)); 
    for i=1:n_w
        pv = gcm_get_partitions(Wref(i,:));
        namestr = sprintf('w(%d)',i);
        displine(namestr,pv);
    end
    
    disp(sprintf('Outputs y: M=%d',M)); 
    for i=1:n_y
        pv = gcm_get_partitions(Yref(i,:));
        namestr = sprintf('y(%d)',i);
        displine(namestr,pv);
    end
    
    [YN,In]=gcm_check_for_grid(Xref);
    if YN, fprintf('States x (grid): S = %d',S);
    else, fprintf('States x: S = %d',S);
    end
    if ~all(all(isfinite(Xref))), fprintf(' incl. sink cell\n');, else, fprintf('\n'); end
    if YN
        if YN, [tmp,lb,ub]=gcm_x2s(Xref(:,1),Xref,In); end
        for i=1:n_x
            pv = gcm_get_partitions(Xref(i,:));
            namestr = sprintf('x(%d)',i);
            displine2(namestr,pv,lb(i),ub(i));
        end
    else
        for i=1:n_x
            pv = gcm_get_partitions(Xref(i,:));
            namestr = sprintf('x(%d)',i);
            displine(namestr,pv);
        end
    end
    
    disp(sprintf('Controls u: A=%d',A)); 
    for i=1:n_u
        pv = gcm_get_partitions(Uref(i,:));
        namestr = sprintf('u(%d)',i);
        displine(namestr,pv);
    end
    
    disp('============================================================')

    if isfield(STR,'f')
        for a=1:A
            N(a,1:3) = full([min(STR.f(a).Nsum) mean(STR.f(a).Nsum) max(STR.f(a).Nsum)]);
        end
        if all(N==N(1,1));
            disp(sprintf('N=%d evaluations on all state-action pairs.',N(1,1)))
        else
            disp(sprintf('N=[%d,%d] evaluations on state-action pairs.',min(min(N)),max(max(N))))
        end
    end
    
    disp('============================================================')
    
elseif ISCOST
    
    Q = length(STR);
    [S,A] = size(STR(1).R);
    if nargout, return; end

    disp('==== CONTROLLER INFO ==================================')
    fprintf('A DSS control system with:\n   %d states\n   %d actions\n   %d set points\n',S,A,Q)
    if isfield(STR(1),'controllertype')
        if strcmp(STR(1).controllertype,'predictive')
            fprintf('   %s control design, H_p=%g',STR(1).controllertype,STR(1).H_p)
        elseif strcmp(STR(1).controllertype,'optimal')
            fprintf('   %s control design, gamma=%g (H=%g)',STR(1).controllertype,STR(1).gam,1/(1-STR(1).gam))
        else
            fprintf('unknown control design')
        end
    else
        fprintf('controller type not specified')
    end
    %fprintf(' in:\n      ''%s''\n',STR(1).control_setup_file)
    fprintf('\n');

    disp('=======================================================')
    for q=1:Q
        if q==1,fprintf('Immediate costs in R: min          mean         max\n'); end
        fprintf('q =%2d: ',q);
        %        fprintf(' Immediate costs in R:  %g < %g < %g\n',min(min(STR(q).R)),mean(mean(STR(q).R)),max(max(STR(q).R)))
        fprintf('            %10.5f < %10.5f < %10.5f\n',min(min(STR(q).R)),mean(mean(STR(q).R)),max(max(STR(q).R)))
    end
    disp('=======================================================')
    for q=1:Q
        if q==1,fprintf('Expected costs in J*: min          mean         max\n'); end
        fprintf('q =%2d: ',q);
        %        fprintf(' Immediate costs in R:  %g < %g < %g\n',min(min(STR(q).R)),mean(mean(STR(q).R)),max(max(STR(q).R)))
        if isfield(STR(q),'Jstar')
            %fprintf('        Expected costs in J*:  %g < %6g < %g\n',min(STR(q).Jstar),mean(STR(q).Jstar),max(STR(q).Jstar))        
            fprintf('            %10.5f < %10.5f < %10.5f\n',min(STR(q).Jstar),mean(STR(q).Jstar),max(STR(q).Jstar))        
        end
    end
    disp('=======================================================')
end

if nargout==0, clear S; end



% ---------------------------------------------------------------------
function displine(namestr,pv)
pv = pv(isfinite(pv));
lpv = length(pv);
switch(lpv)
case 1
    str = sprintf('   %s: {%g}   (%g)',namestr,pv(1),lpv);
case 2
    str = sprintf('   %s: {%g, %g}   (%g)',namestr,pv(1),pv(2),lpv);
case 3
    str = sprintf('   %s: {%g, %g, %g}   (%g)',namestr,pv(1),pv(2),pv(3),lpv);
case 4
    str = sprintf('   %s: {%g, %g, %g, %g}   (%g)',namestr,pv(1),pv(2),pv(3),pv(length(pv)),lpv);
case 5
    str = sprintf('   %s: {%g, %g, %g, %g, %g}   (%g)',namestr,pv(1),pv(2),pv(3),pv(4),pv(length(pv)),lpv);
otherwise
    str = sprintf('   %s: {%g, %g, ..., %g, %g}   (%g)',namestr,pv(1),pv(2),pv(length(pv)-1),pv(length(pv)),lpv);
end
disp(str);

function displine2(namestr,pv,lb,ub)
pv = pv(isfinite(pv));
lpv = length(pv);
str0 = sprintf('   %s:',namestr);
str1 = sprintf(' [%g, %g], ',lb,ub);
switch(lpv)
case 1
    str = sprintf('{%g}',pv(1));
case 2
    str = sprintf('{%g, %g}',pv(1),pv(2));
case 3
    str = sprintf('{%g, %g, %g}',pv(1),pv(2),pv(3));
case 4
    str = sprintf('{%g, %g, %g, %g}',pv(1),pv(2),pv(3),pv(length(pv)));
case 5
    str = sprintf('{%g, %g, %g, %g, %g}',pv(1),pv(2),pv(3),pv(4),pv(length(pv)));
otherwise
    str = sprintf('{%g, %g, ..., %g, %g}',pv(1),pv(2),pv(length(pv)-1),pv(length(pv)));
end
strf = sprintf(' (%d)',lpv);
disp(sprintf('%10s %20s %20s %5s',str0,str1,str,strf));
