function Cost = gcmoptipol(Plant,Cost,q)
% GCMOPTIPOL. Solve optimal policy
%    Cost = gcmoptipol(Plant,Cost,q)
%    Cost = gcmoptipol(Plant,Cost)
%

% Enso 11/2006

   
%%%%%%%%%%%%
CRI = 1/100;        % 1 percent accuracy required
%CRI = 1/10000;       % 0.01% accuracy

% Read arguments
S = size(Plant.Xref,2);
A = size(Plant.Uref,2);
Q = size(Plant.Wref,2);

if nargin<3
    for q=1:Q
        Cost = gcmoptipol(Plant,Cost,q);
    end
    return;
end

% Display problem size
disp(sprintf('GCMOPTIPOL: Solving a %dx%d Bellman eq. (q=%d).',S,A,q))

% Step 1. Initialize
INIT=1;
if isfield(Cost(q),'Jstar')
    if ~isempty(Cost(q).Jstar)
        Jstar = Cost(q).Jstar;
        INIT=0;
    end
end
if INIT, 
    Jstar = zeros(S,1); % Jstar = spalloc(S,1,S); 
    % verify for common mistakes
    [S2,S3]=size(Plant.f(A).P);
    if any([S2 S3]-S)
        error('GCMOPTIPOL: Suspicious matrix sizes')
    end
end

k = 0;
% Step 2. Iterate
t0 = clock; t00=t0; DISPL=0; ZZ = -1;
Q = zeros(S,A); %Q = spalloc(S,A,S*A);
while 1
    k = k+1;
    Jstar_old = Jstar;

    % for s=1:S, for a=1:A, Q(s,a) = Cob.R(s,a) + Cob.gam * Mo(a).p(s,:) * Jstar_old; end, [Jstar(s),Pistar(s)] = min(Q(s,:)); end
    for a=A:-1:1
        % Bellman update
        % This computation is crucial:
        % Q(:,a)            [S,1]
        % Cost(q).R(:,a)    [S,1]
        % Cost(q).gam       [1,1]
        % Plant.f(a).P      [S,S]   must be sparse due to memory requirements
        % Jstar_old         [S,1]
        % compute PJ* first (result is sparse)
        % convert to full, and the rest of the comp. are full
        Q(:,a) = Cost(q).R(:,a) + Cost(q).gam * Plant.f(a).P' * Jstar_old;
    end
    % Bellman equation
    [Jstar,Pistar] = min(Q');
    Jstar=Jstar';
   
    % Terminating criteria
    %cri = max( (Jstar_old-Jstar)./(Jstar_old+1000*eps) );
    cri = max(abs( (Jstar_old-Jstar)./(mean(Jstar_old)+100*eps) ));
    % cri = max(abs( (Jstar_old-Jstar) ));
    %cri = full(max( (Jstar_old-Jstar)./(Jstar_old+1000*eps) ));
    %cri = full(max(abs( (Jstar-Jstar_old)./(Jstar+1000*eps) )));
    
    % termination & display stuff..
    if k==1 & DISPL==0 
        fprintf('   Value iteration: until max | [J*(k)-J*(k-1)]/mean(J(k-1))] | < %1.0g%%: ',100*CRI);             
        DISPL=1;
        t0=clock; 
    end
    if cri<CRI
        if DISPL, fprintf('%3.3f%% ok.\n',100*cri); end
        break;
    elseif etime(clock,t00)>15
        if DISPL
            fprintf('%1.0f%% ',100*cri); t00=clock; ZZ=ZZ+1; 
            if ZZ==15; fprintf('\n   '); ZZ = 0; end
        end
        %break;
    end

    %JLOG = [JLOG;Jstar'];
    %if etime(clock,t0)>5, plot(JLOG); t0=clock; ff; end
    %S,etime(clock,t0), t0 = clock;
end

% 3 Solve optimal policy
[Jstar,Pistar] = min(Q');
Jstar=Jstar';
if 0
    for s = 1:S
        % In case of several optimal actions, 
        % take randomly one action among the optimal ones
        Jstar(s) = min(Q(s,:));
        tmp = find(Q(s,:)==Jstar(s));
        Pistar(s) = gcm_automeq(tmp);
    end
end

% Set output
Cost(q).Pi = Pistar;
Cost(q).Jstar = Jstar;


