function a = gcmpred_online(p_x,Plant,Cost,q)
% GCMPRED_ONLINE Compute predictive controller output on-line
%
%   a = gcmpred_online(p_x,Plant,Cost,q)   for q'th set point
%
% reads:
%   p_x           distribution (estimate) for current state
%   Cost(q).H_p   prediction horizon [1,1]
%   Cost(q).R     (immediate) costs [S,A]
%   Plant.f(a).P  probability transition matrices for each a=1,2,...,A [S,S]
%   q             index to controller set point, Plant.Wref(:,q)
% returns:
%   a             control action to be applied to plant

t00 = clock;

% solve states for prediction horizon
[S,A] = size(Cost(1).R);
Q = length(Cost);

% set list of q's (set point indexes)
qlist = q;

%J = zeros(S,lql*A);     % initialize cost-matrix
for q=qlist, tmp(q).J = zeros(A,1); end     % initialize temporary cost-matrices
maxH_p = max([Cost(qlist).H_p]);
str = sprintf('%d,',qlist); str = str(1:length(str)-1);
fprintf('\nGCM_PRED_ONLINE: Evaluating set point q=%s for H_p = %d\n   ',str,maxH_p)

CNT = 1; t0 = clock; ss = 0;
% Compute A state trajectories (control horizon H_c=1)
for a=1:A

    % initial state
    pp = p_x;  
    % compute trajectory in prediction horizon
    for h=0:maxH_p
        % evaluate costs at k+h, for each set point q and action a
        for q=qlist
            if Cost(q).H_p>=h   % q's may have different H_p's
                %J(:,ia) = J(:,ia) + pp'*Cost(q).R(:,a);     % sum of (immediate) costs in prediction horizon
                tmp(q).J(a,:) = tmp(q).J(a,:) + Cost(q).R(:,a)'*pp;     % sum of (immediate) costs in prediction horizon
            end
        end
        % quit
        if h==maxH_p, break; end
        
        pp = Plant.f(a).P*pp;   
    end
    
    % disp
    if CNT<4 | etime(clock, t0)>15
        if rem(CNT,10)==0, fprintf('\n   '); end
        CNT = CNT+1;
        fprintf(' %d%%',round(100*a/A) );
        t0 = clock;
    end
    
end

% find optimal actions
for q = qlist
    [Jmin,a] = min(tmp(q).J);
end

str = gcm_sec2time(etime(clock,t00));
fprintf(' 100%% (%s) ok. ',str);
