% GCMRUN    Script for running successive model-updates
%           Runs model updates one by one, updates controller every now and then, 
%           simulates and plots a figure, and stores model on disk.
%
%           Set EVA to max number of model update iterations (default EVA=500)

if ~exist('EVA')
    EVA = 500;  % 1/EVA is the percent-unit accuracy for each pdf (A open-loop models)
end

if isfield(Plant,'f')
    eva = Plant.f(1).Nsum(1);
end

while 1
    disp(sprintf('\n\n=== %s === EVA = %d of %d: ===============',datestr(now),eva+1,EVA))
    
    % MODEL CONVERSION
    Plant = gcm_generate_model(Plant);
    eva = eva+1;
    
    % MODEL ANALYSIS, CONTROL DESIGN AND ANALYSIS
    if eva==1 | eva==2 | eva==5 | eva==10 | eva==50 | rem(eva,100)==0
               
        % Design a contoller (script for updating Cost-structure)
        if exist('Cost','var')==1
            if isfield(Cost,'gam')
                Cost = gcmoptipol(Plant,Cost);
            elseif isfield(Cost,'H_p')
                Cost = gcmpredpol(Plant,Cost);
            end
        end
    
        % simulate closed-loop
        if exist('Sim','var')==1
            if isfield(Sim,'WW')
                Sim=gcmsim(Plant,Cost,Sim);
            end
        end
    end
    
    % SAVE   
    save gcmrun Plant Cost Sim EVA
    disp('Saved to gcmrun.mat')
    if eva==0 | eva==1 | eva==2 | eva==5 | (eva<100 & rem(eva,10)==0) | rem(eva,100)==0
        str = sprintf('save test_hybrid_%d Plant Cost Sim',eva);
        eval(str);
    end
    
    if eva==EVA, break; end
end