function unravel_disp(Gr,P,St,C,Traj,Plant);
% UNRAVEL_DISP Plot results of the unravelling algorithm
%
%   unravel_disp(Gr,P,St,C,Traj,Plant);


% ===================
% = Display results =
% ===================
% ===================
if nargin>4
    OUTPUTTRAJECTORIES=1;
else
    OUTPUTTRAJECTORIES=0;
end
%figure(1)
%subplot(221); bar(C); ylabel('C')
%subplot(222); bar(Gr); ylabel('Group number')
%subplot(223); bar(P); ylabel('Periodicity number')
%subplot(224); bar(St); ylabel('Step number')
%for sp=1:4, subplot(2,2,sp); ax=axis; axis([0 length(C)+1 ax(3:4)]); end
%ff;

% simulate longest and shortest trajectories in each group
N = length(Gr); % # cells
Gtot = max(Gr);    % # groups
G = min(1,Gtot);
K = max(St)+ceil(max(St)/3);    % length of simulation
fprintf('Simulating trajectories for %d groups...\n',G); 

Traj = zeros(K,2*G);
t0 = clock;
for s=1:G
    tmp = find(Gr==s);
    e_max = tmp(find(St(tmp)==max(St(tmp))));
    e_max = e_max(1);
    e_min = tmp(find(St(tmp)==min(St(tmp))));
    e_min = e_min(1);
    for k=1:K
        Traj(k,s)=e_min;
        e_min=C(e_min);
        Traj(k,s+G)=e_max;
        e_max=C(e_max);
    end
    if etime(clock, t0)>5, fprintf('%d%%-',round(100*s/G)); t0 = clock; end
end
fprintf('ok.\n')

% plot
kk=[1:K]'-1;
if OUTPUTTRAJECTORIES
    % Replace state-trajecories Traj by output trajectories
    Traj = gcm_ytraj(Traj,Plant,1);
end
h=plot(kk,Traj(:,1:G),'-',kk,Traj(:,G+[1:G]),'-',kk,Traj(:,G+[1:G]),'.');    

% set colors and linestyles
i = [1:64]'; ii = [64:-1:1]'; i1 = (1/64)*i; ii1 = (1/64)*ii; i0 = zeros(64,1);
icoe = 1+4*(i1.*(1-i1));
icoe2 = 1-exp(-[0:63]');
RBmap = [i1 i0 ii1];   % 64 colors, most important at 64
RBmap(:,1) = icoe.*RBmap(:,1).*icoe2;
RBmap(:,3) = icoe.*RBmap(:,3).*icoe2;
RBmap = min(RBmap,1);
for s=1:G
    idx = ceil(64*sum(Gr==s)/N);
    %col = get(h(s),'color');
    col = RBmap(idx,:);
    set(h(s),'color',col);
    set(h(G+s),'color',col);
    set(h(2*G+s),'color',col);
    text(K-0.5,Traj(K,s),num2str(s),'color',col);
    set(h(s),'linewidth',max(0.25,5*idx/64));
    %colormap(CM);
end
% set axes
xlabel('time')
if OUTPUTTRAJECTORIES
    ytic = sort(Plant(1).Wref);
    ylabel('Output');
    ax = axis; axis([-1 K min(Plant.Yref) max(Plant.Yref)]);
else
    ytic = gcm_get_partitions(Traj(K,1:G));
    if ytic(1)~=1, ytic=[1 ytic]; end
    if ytic(length(ytic))~=N, ytic = [ytic N]; end
    ylabel('State');	
    %ax = axis; axis([-1 K 0 N+1]);
end
%et(gca,'ytick',ytic);

% make legend
str = 'legend(';
for s=1:min(10,G)
    c = find(Gr==s);
    stri = sprintf('Gr%d: P-%d (doa %d)',s,P(c(1)),length(c));
    str = sprintf('%s''%s'',',str,stri);
end
str=[str,'-1);'];
eval(str);

shg;
%pause;

