function Plant = vdv5_plant_setup
% VDV_PLANT_SETUP   Sample plant setup file for initialization of Plant structure.
%   Plant = fun_plant_setup
% where
% Plant is a structre where fhe following fields are set:
%   funf    name of the DSS (discrete-time state space) system equation m-file
%   funh    name of the measurement equation m-file
%   Ts      system sampling time
%   Xref    state discretization (reference points)
%   Uref    control discretization (action space)
%   Yref    output discretization (measurement reference points)
%   Wref    set point discretization

% See VDVF, VDVH

% DSS system info
% ---------------
funf_str = 'vdvf5'  % name of m-file containing system equations x(k+1) = f(x(k),u(k))
funh_str = 'vdvh'   % name of m-file containing measurement equations y(k) = h(x(k))
Ts = eval(funf_str);          % read Ts from FUNF
Plant.funf = funf_str;
Plant.funh = funh_str;
Plant.Ts = Ts;

% Discretize variables:
% ---------------------
% read info from FUNF and FUNH
[Ts,xn,un,xmin,xmax,umin,umax,xname,uname] = eval(funf_str);  % read info from FUNF
[yn,ymin,ymax,yname] = eval(funh_str);  % read info from FUNH
Ts

% Write variable names to Plant-structure
Plant.Xname = xname;
Plant.Uname = uname;
Plant.Yname = yname;
Plant.Wname = yname;

% read sizes of variables
n_x = size(xn,1);
n_u = size(un,1);
n_y = size(yn);
n_w = n_y;

% Discretize spaces
dx1 = 0.1; dx2 = 0.04;
%dx1 = 0.1; dx2 = 0.04;
x_part{1} = [1 1.3 1.5:dx1:2.5 2.7 2.9 3.1 3.4];      % c_A concentration of comp A
x_part{2} = [0.6 0.7  [0.76:dx2:1.14] 1.2 1.3];  % c_B concentration of comp B (product)
x_part{3} = [90 [100:5:115] 125];    % v reactor temp
x_part{4} = [90 [100:5:115] 125];    % v_K cooler temp
x_part{5} = [[100:5:115]];    % v_0 feed temp
du1 = 1;
u_part{1} = [3 6 9 11 13 14 15 17 20 25 30 35];
u_part{2} = [-9000 -3000 -1113.5 -800 0];
y_part{1} = x_part{2};
w_part{1} = [0.8 0.95 1.05 1.09];    

% generate cmd-string for using 'gcm_inigrid', evaluate

% Define Xref
cmd = 'gcm_inigrid(';
for i=1:n_x, cmd = [cmd,sprintf('x_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Xref = eval(cmd);

% Define Uref
cmd = 'gcm_inigrid(';
for i=1:n_u, cmd = [cmd,sprintf('u_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Uref = eval(cmd);

% Define Yref
cmd = 'gcm_inigrid(';
for i=1:n_y, cmd = [cmd,sprintf('y_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Yref = eval(cmd);

% Define Wref
cmd = 'gcm_inigrid(';
for i=1:n_w, cmd = [cmd,sprintf('w_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Wref = eval(cmd);

% Add sink cells
if 1
    n_x = size(Xref,1);
    Xref = [Xref Inf*ones(n_x,1)];  
    n_y = size(Yref,1);
    Yref = [Yref Inf*ones(n_y,1)];  
    disp('Sink cell added');
end

% write to Plant
Plant.Xref = Xref;
Plant.Uref = Uref;
Plant.Yref = Yref;
Plant.Wref = Wref;

% === SUBFUN EQSPACE ======================
function x=eqspace(xmin,xmax,n)
if n>1
    x=[xmin:(xmax-xmin)/(n-1):xmax];
elseif n==1
    x=(xmin+xmax)/2;
else
    error('EQSPACE: Must have n>0')
end
