% VDV5_TEST  A sample script for running CFMC-Toolbox

if ~exist('Plant','var')
    
    % Initialize Plant-structure model-setup
    % --------------------------------------
    % --- MODIFY HERE 1/4 --------
    % - uses system DSS-model defined in state and measurement equation M-files
    Plant = vdv5_plant_setup;
    gcmdisp(Plant)
    
    % Example model generation + analysis
    % -----------------------------------
    for j=1:1, Plant = gcm_generate_model(Plant); end
    
    % nominal cell and action
    % --- MODIFY HERE 2/4 --------
    [Ts,Xnom,Unom] = vdvf5; 
    snom = gcm_x2s(Xnom,Plant.Xref); anom = gcm_x2s(Unom,Plant.Uref);
    % simulate pdf trajectories
    gcm_step(Plant,snom,anom)
    % analysis of cell classes..
    % [EA,Et_t2r,Et_t2r,Et_stat]=gcm_eiganalysis(Plant,anom);
    % CClass=gcm_comm(Plant,anom);
    
    clear Cost Sim
end

if ~exist('Cost','var')
    % Set controller specifications
    % -----------------------------
    % --- MODIFY HERE 3/4 --------
    Cost = fun_control_setup(Plant);
    % Set simulation specifications
    Plant.simf = Plant.funf; 
    Plant.simh = Plant.funh; 
    clear Sim
end

if ~exist('Sim','var')
    % Set simulation parameters
    % --- MODIFY HERE 4/4 --------
    Sim = fun_simulation_setup(Plant);
end

if ~(exist('e') & exist('E') & exist('RRRR'))
    e = 0;
    E = 1000; RRRR=[];
end

% Model building / controller solving -loop:
% ==========================================
tss = clock;
while e<E
    
    % Solve for the controller
    if isfield(Cost,'gam')
        Cost = gcmoptipol(Plant,Cost);
    elseif isfield(Cost,'H_p')
        Cost = gcmpredpol(Plant,Cost);
    else
        error('Unknown controller')
    end
    
    % Simulate closed-loop
    if etime(clock, tss)>5 | e<10 | e==E
        % Simulate
        Sim = gcmsim(Plant,Cost,Sim);
        subplot(111); title(sprintf('iteration %d/%d RMSE=%f',e,E,Sim.rmse_rd(1))); drawnow;
        gcf;
        tss = clock;
    end
    RRRR = [RRRR;Sim.rmse_rd];
    figure(2); clf; plot(RRRR); shg; figure(1);
    
    % Make the model more precise
    for j=1:1
        Plant = gcm_generate_model(Plant);
        e = e+1;
    end 
end